@@ -11,6 +11,7 @@ use core::iter::FusedIterator;
 use core::marker::{PhantomData, PhantomPinned};
 use core::mem::{self, ManuallyDrop, MaybeUninit};
 use core::pin::Pin;
+use core::ptr;
 use core::slice;
 
 /// Binding to C++ `std::vector<T, std::allocator<T>>`.
@@ -53,6 +54,15 @@ where
         T::__vector_size(self)
     }
 
+    /// Returns the capacity of the vector.
+    ///
+    /// Matches the behavior of C++ [std::vector\<T\>::capacity][capacity].
+    ///
+    /// [capacity]: https://en.cppreference.com/w/cpp/container/vector/capacity
+    pub fn capacity(&self) -> usize {
+        T::__vector_capacity(self)
+    }
+
     /// Returns true if the vector contains no elements.
     ///
     /// Matches the behavior of C++ [std::vector\<T\>::empty][empty].
@@ -74,6 +84,10 @@ where
 
     /// Returns a pinned mutable reference to an element at the given position,
     /// or `None` if out of bounds.
+    ///
+    /// This method cannot be named "get\_mut" due to a conflict with
+    /// `Pin::get_mut`.
+    #[doc(alias = "get_mut")]
     pub fn index_mut(self: Pin<&mut Self>, pos: usize) -> Option<Pin<&mut T>> {
         if pos < self.len() {
             Some(unsafe { self.index_unchecked_mut(pos) })
@@ -93,9 +107,9 @@ where
     ///
     /// [operator_at]: https://en.cppreference.com/w/cpp/container/vector/operator_at
     pub unsafe fn get_unchecked(&self, pos: usize) -> &T {
-        let this = self as *const CxxVector<T> as *mut CxxVector<T>;
+        let this = ptr::from_ref::<CxxVector<T>>(self).cast_mut();
         unsafe {
-            let ptr = T::__get_unchecked(this, pos) as *const T;
+            let ptr = T::__get_unchecked(this, pos).cast_const();
             &*ptr
         }
     }
@@ -111,6 +125,10 @@ where
     /// [std::vector\<T\>::operator\[\]][operator_at].
     ///
     /// [operator_at]: https://en.cppreference.com/w/cpp/container/vector/operator_at
+    ///
+    /// This method cannot be named "get\_unchecked\_mut" due to a conflict with
+    /// `Pin::get_unchecked_mut`.
+    #[doc(alias = "get_unchecked_mut")]
     pub unsafe fn index_unchecked_mut(self: Pin<&mut Self>, pos: usize) -> Pin<&mut T> {
         unsafe {
             let ptr = T::__get_unchecked(self.get_unchecked_mut(), pos);
@@ -133,7 +151,7 @@ where
             // which upholds the invariants.
             &[]
         } else {
-            let this = self as *const CxxVector<T> as *mut CxxVector<T>;
+            let this = ptr::from_ref::<CxxVector<T>>(self).cast_mut();
             let ptr = unsafe { T::__get_unchecked(this, 0) };
             unsafe { slice::from_raw_parts(ptr, len) }
         }
@@ -196,6 +214,51 @@ where
             })
         }
     }
+
+    /// Ensures that this vector's capacity is at least `additional` elements
+    /// larger than its length.
+    ///
+    /// The capacity may be increased by more than `additional` elements if the
+    /// implementation chooses, to amortize the cost of frequent reallocations.
+    ///
+    /// **The meaning of the argument is not the same as
+    /// [std::vector\<T\>::reserve][reserve] in C++.** The C++ standard library
+    /// and Rust standard library both have a `reserve` method on vectors, but
+    /// in C++ code the argument always refers to total capacity, whereas in
+    /// Rust code it always refers to additional capacity. This API on
+    /// `CxxVector` follows the Rust convention, the same way that for the
+    /// length accessor we use the Rust conventional `len()` naming and not C++
+    /// `size()`.
+    ///
+    /// # Panics
+    ///
+    /// Panics if the new capacity overflows usize, or if `T` is not
+    /// move-constructible in C++.
+    ///
+    /// [reserve]: https://en.cppreference.com/w/cpp/container/vector/reserve.html
+    pub fn reserve(self: Pin<&mut Self>, additional: usize) {
+        let new_cap = self
+            .len()
+            .checked_add(additional)
+            .expect("CxxVector capacity overflow");
+        unsafe { T::__reserve(self, new_cap) }
+    }
+}
+
+impl<T> Extend<T> for Pin<&mut CxxVector<T>>
+where
+    T: ExternType<Kind = Trivial> + VectorElement,
+{
+    fn extend<I>(&mut self, iter: I)
+    where
+        I: IntoIterator<Item = T>,
+    {
+        let iter = iter.into_iter();
+        self.as_mut().reserve(iter.size_hint().0);
+        for element in iter {
+            self.as_mut().push(element);
+        }
+    }
 }
 
 /// Iterator over elements of a `CxxVector` by shared reference.
@@ -279,7 +342,7 @@ where
         // Extend lifetime to allow simultaneous holding of nonoverlapping
         // elements, analogous to slice::split_first_mut.
         unsafe {
-            let ptr = Pin::into_inner_unchecked(next) as *mut T;
+            let ptr = ptr::from_mut::<T>(Pin::into_inner_unchecked(next));
             Some(Pin::new_unchecked(&mut *ptr))
         }
     }
@@ -350,8 +413,12 @@ pub unsafe trait VectorElement: Sized {
     #[doc(hidden)]
     fn __vector_size(v: &CxxVector<Self>) -> usize;
     #[doc(hidden)]
+    fn __vector_capacity(v: &CxxVector<Self>) -> usize;
+    #[doc(hidden)]
     unsafe fn __get_unchecked(v: *mut CxxVector<Self>, pos: usize) -> *mut Self;
     #[doc(hidden)]
+    unsafe fn __reserve(v: Pin<&mut CxxVector<Self>>, new_cap: usize);
+    #[doc(hidden)]
     unsafe fn __push_back(v: Pin<&mut CxxVector<Self>>, value: &mut ManuallyDrop<Self>) {
         // Opaque C type vector elements do not get this method because they can
         // never exist by value on the Rust side of the bridge.
@@ -422,6 +489,13 @@ macro_rules! impl_vector_element {
                 }
                 unsafe { __vector_size(v) }
             }
+            fn __vector_capacity(v: &CxxVector<$ty>) -> usize {
+                extern "C" {
+                    #[link_name = concat!("cxxbridge1$std$vector$", $segment, "$capacity")]
+                    fn __vector_capacity(_: &CxxVector<$ty>) -> usize;
+                }
+                unsafe { __vector_capacity(v) }
+            }
             unsafe fn __get_unchecked(v: *mut CxxVector<$ty>, pos: usize) -> *mut $ty {
                 extern "C" {
                     #[link_name = concat!("cxxbridge1$std$vector$", $segment, "$get_unchecked")]
@@ -429,6 +503,13 @@ macro_rules! impl_vector_element {
                 }
                 unsafe { __get_unchecked(v, pos) }
             }
+            unsafe fn __reserve(v: Pin<&mut CxxVector<$ty>>, new_cap: usize) {
+                extern "C" {
+                    #[link_name = concat!("cxxbridge1$std$vector$", $segment, "$reserve")]
+                    fn __reserve(_: Pin<&mut CxxVector<$ty>>, _: usize);
+                }
+                unsafe { __reserve(v, new_cap) }
+            }
             vector_element_by_value_methods!($kind, $segment, $ty);
             fn __unique_ptr_null() -> MaybeUninit<*mut c_void> {
                 extern "C" {
